/*
 *  Copyright (c) Members of the EGEE Collaboration. 2005.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Management - Simple channel management API
 *
 *  Authors: Gabor Gombas <Gabor.Gombas@cern.ch>
 *
 */

#ifndef GLITE_DATA_TRANSFER_CHANNEL_SIMPLE_H
#define GLITE_DATA_TRANSFER_CHANNEL_SIMPLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <glite/data/transfer/c/transfer-simple.h>

/**********************************************************************
 * Constants
 */

/* Service type for Service Discovery */
#define GLITE_CHANNEL_SD_TYPE		"org.glite.ChannelManagement"

/* Environment variable to override service type */
#define GLITE_CHANNEL_SD_ENV		"GLITE_SD_CHANNEL_TYPE"

/**********************************************************************
 * Function prototypes - library management functions
 */

/**
 * @defgroup transfer_lib_management Transfer related management functions
 * @{
 */
		
/**
 * Get the current endpoint used by the transfer context.
 *
 * @param ctx The transfer context.
 *
 * @return The string containing the endpoint.
 */
const char *glite_channel_get_endpoint(glite_transfer_ctx *ctx);

/**
 * Converts the state-string to the proper ENUM value.
 * 
 * @param ctx The transfer context. Only used for error handling.
 * @param state The state string to be converted.
 * 
 * @return The ::glite_transfer_state type variable. NULL if there was an error.
 */
glite_channel_state glite_channel_state_parse(glite_transfer_ctx *ctx,
	const char *state);

/**
 * Converts the transfer-state variable to a human readable string.
 * 
 * @param state The varible to be converted.
 *
 * @return The string containing the name of the state. "Unknown" in case the variable is not the name of a possible state.
 */
const char *glite_channel_state_str(glite_channel_state state);

/**
 * @}
 */

/**********************************************************************
 * Function prototypes - org.glite.data.transfer.ServiceBase
 */

/**
 * @defgroup ServiceBase org.glite.data.transfer.ServiceBase
 * @{
 */

/**
 * Get the version of the remote endpoint.
 * The returned string must be freed by the caller. 
 * 
 * @param ctx The transfer context for error handling.
 *
 * @return The version string. In case of an error NULL is returned.
 */
char *glite_channel_getVersion(glite_transfer_ctx *ctx);

/**
 * Get the schema version of the remote endpoint.
 * The returned string must be freed by the caller.
 * 
 * @param ctx The transfer context for error handling.
 *
 * @return The schema-version string. In case of an error NULL is returned.
 */
char *glite_channel_getSchemaVersion(glite_transfer_ctx *ctx);

/**
 * Get the interface version of the remote endpoint.
 * The returned string must be freed by the caller.
 * 
 * @param ctx The transfer context for error handling.
 *
 * @return The interface-version string. In case of an error NULL is returned.
 */
char *glite_channel_getInterfaceVersion(glite_transfer_ctx *ctx);

/**
 * Get the service metadata for a given key. The key must not be NULL.
 * The returned string must be freed by the caller.
 * 
 * @param ctx The transfer context for error handling.
 * @param key The key of which the value we want.
 *
 * @return The interface-version string. In case of an error NULL is returned. 
 */
char *glite_channel_getServiceMetadata(glite_transfer_ctx *ctx, const char *key);

/**
 * @}
 */

/**********************************************************************
 * Function prototypes - org.glite.data.transfer.channel.ChannelManagement
 */

/** @defgroup transfer_channel org.glite.data.transfer.channel.ChannelManagement
 * @{
 */

/**
 * Add a new transfer channel to the database.
 * 
 * @param ctx The transfer context. 
 * @param channel The existing channel object to be added.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_add(glite_transfer_ctx *ctx,
	const glite_transfer_Channel *channel);

/**
 * Drop a channel from the database.
 * 
 * @param ctx The transfer context. 
 * @param name The name of the channel to be dropped. 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_drop(glite_transfer_ctx *ctx, const char *name);

/**
 * Get a channel object by its name.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel.
 *
 * @return The transfer-channel object, or NULL if there was an error.
 */
glite_transfer_Channel *glite_channel_getChannel(glite_transfer_ctx *ctx,
	const char *name);

glite_transfer_Channel2 *glite_channel_getChannel2(glite_transfer_ctx *ctx,
   const char *name);

/**
 * List all channels in the database.
 * 
 * @param ctx The transfer context.
 * @param resultCnt [OUT] The number of items in the return array.
 *
 * @return A list containing the names of the channels, or NULL if there was an error.
 */
char **glite_channel_listChannels(glite_transfer_ctx *ctx, int *resultCnt);

/**
 * Manual intervention command on channel.
 * It changes the file_state for all files belonging to the given job from Hold to the
 * given file state. It will only work for files in the Hold state. The only permissible
 * requested states are 'Pending', 'Failed' or 'Canceling'.
 * 
 * @param ctx The transfer context.
 * @param jobId The ID of the job.
 * @param state The new state.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_changeStateForHeldJob(glite_transfer_ctx *ctx,
	const char *jobId, glite_transfer_state state);

/**
 * Manual intervention command on channel.
 * It changes the file_state for all files belonging to all jobs on the given channel
 * which are in the Hold state to the given file state. It will only work for files
 * in the Hold state. The only permissible requested states are 'Pending', 'Failed'
 * or 'Canceling'.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel.
 * @param state The new state.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_changeStateForHeldJobsOnChannel(glite_transfer_ctx *ctx,
	const char *name, glite_transfer_state state);

/**
 * Set the theoretical bandwidth of a channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel.
 * @param utilisation The new bandwidth value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setBandwidth(glite_transfer_ctx *ctx, const char *name,
	int utilisation);
int glite_channel_setBandwidth2(glite_transfer_ctx *ctx, const char *name,
	int utilisation, const char* message);

/**
 * Set the contact information of a channel which can be queried in case of a problem.
 * 
 * @param ctx The transfer context. 
 * @param name The name of the channel.
 * @param contact The contact info string.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setContact(glite_transfer_ctx *ctx, const char *name,
	const char *contact);
int glite_channel_setContact2(glite_transfer_ctx *ctx, const char *name,
	const char *contact, const char* message);

/**
 * Set the default TCP buffer size for a channel.
 * 
 * @param ctx The transfer context. 
 * @param name The name of the channel.
 * @param buffer_size The TCP buffer size string.
 *
 * @return 0 when successful and -1 in case of an error.
 */

int glite_channel_setTCPBufferSize(glite_transfer_ctx *ctx, const char *name,
	const char *buffer_size);
int glite_channel_setTCPBufferSize2(glite_transfer_ctx *ctx, const char *name,
	const char *buffer_size, const char* message);

/**
 * Set the maximal throughput value of a channel which should not be overrun.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel.
 * @param nominalThroughput The new throughput value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setNominalThroughput(glite_transfer_ctx *ctx,
	const char *name, int nominalThroughput);
int glite_channel_setNominalThroughput2(glite_transfer_ctx *ctx,
	const char *name, int nominalThroughput, const char* message);

/**
 * Set the maximal number of files that can be transfered simultanously on a channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel. 
 * @param numberOfFiles The new value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setNumberOfFiles(glite_transfer_ctx *ctx, const char *name,
	int numberOfFiles);
int glite_channel_setNumberOfFiles2(glite_transfer_ctx *ctx, const char *name,
	int numberOfFiles, const char* message);

/**
 * Set the maximal number of streams a file transfer can be split.
 * @param ctx The transfer context.
 * @param name The name of the channel.
 * @param numberOfStreams The new value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setNumberOfStreams(glite_transfer_ctx *ctx, const char *name,
	int numberOfStreams);
int glite_channel_setNumberOfStreams2(glite_transfer_ctx *ctx, const char *name,
	int numberOfStreams, const char* message);

/**
 * Set the state of a channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param state The state to be set. 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setState(glite_transfer_ctx *ctx, const char *name,
	glite_channel_state state);
int glite_channel_setState2(glite_transfer_ctx *ctx, const char *name,
	glite_channel_state state, const char* message);

/**
 * Add a manager to the channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param principal The name of the principal. 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_addManager(glite_transfer_ctx *ctx, const char *name,
	const char *principal);

/**
 * Remove a manager from the channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param principal The name of the principal. 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_removeManager(glite_transfer_ctx *ctx, const char *name,
	const char *principal);

/**
 * List all managers of the channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param resultCnt [OUT] The number of items in the return array.
 *
 * @return A list containing the managers of the channels, or NULL if there was an error.
 */
char **glite_channel_listManagers(glite_transfer_ctx *ctx, const char *name,
        int *resultCnt);

/**
 * Sets the share of a VO on a given channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param VOname The name of the VO
 * @param share The share value
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setVOShare(glite_transfer_ctx *ctx, const char *name,
        const char *VOname, int share);

/**
 * Sets the transfer limit of a VO on a given channel.
 * 
 * @param ctx The transfer context.
 * @param name The name of the channel
 * @param VOname The name of the VO
 * @param limit The limit value
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_channel_setVOLimit(glite_transfer_ctx *ctx, const char *name,
        const char *VOname, int limit);

/**
 * Fetches the channel admin audit for the given period.
 * 
 * @param ctx The transfer context.
 * @param startTime The start time for the audit
 * @param endTime The end time for the audit
 * @param restrictType Check for these events only
 * @param lastNentries Get the last N events
 * @param channelName The channel for which to fetch the audit information
 *
 * @return A list of channel audit entries
 */
glite_transfer_ChannelAuditEntry ** glite_channel_channelAudit(glite_transfer_ctx *ctx, 
         const time_t startTime, const time_t endTime, const char * restrictType,
         const int lastNentries, const char * channelName, int* resultCount);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* GLITE_DATA_TRANSFER_CHANNEL_SIMPLE_H */
